export COCO_DATASETS_PATH=/data/AE/modelzoo/datasets/coco
export ILSVRC2012_DATASETS_PATH=/data/AE/modelzoo/datasets/ILSVRC2012
export SQUAD_DATASETS_PATH=/data/AE/modelzoo/datasets/squadv1.1
export TOTAL_TEXT_DATASETS_PATH=/data/AE/modelzoo/datasets/total_text
export WIDERFACE_DATASETS_PATH=/data/AE/modelzoo/datasets/widerface
export SYNTH_DATASETS_PATH=/data/AE/modelzoo/datasets/Synth90k
export IJB_DATASETS_PATH=/data/AE/modelzoo/datasets/ijb
export VOC2012_DATASETS_PATH=/data/AE/modelzoo/datasets/voc2012/
export CARVANA_DATASETS_PATH=/data/AE/modelzoo/datasets/carvana
export AISHELL_DATASETS_PATH=/data/AE/modelzoo/datasets/data_aishell/
export CITYSCAPES_DATASETS_PATH=/data/AE/modelzoo/datasets/cityscapes
export CHNSENTICORP_DATASETS_PATH=/data/AE/modelzoo/datasets/chnsenticorp
export ICDAR2015_DATASETS_PATH=/data/AE/modelzoo/datasets/ICDAR2015
export UCF101_DATASETS_PATH=/data/AE/modelzoo/datasets/UCF101/
export KINETICS_POSTPROCESS_DATASETS_PATH=/data/AE/modelzoo/datasets/rawframes_val
export KINETICS_DATASETS_PATH=/data/AE/modelzoo/datasets/kinetics/
export CIFAR100_DATASETS_PATH=/data/AE/modelzoo/datasets/cifar100/
export GLUE_DATASETS_PATH=/data/AE/modelzoo/datasets/glue_data
export ROFORMER_DATASETS_PATH=/data/AE/modelzoo/datasets/roformer-sim
export AFLW2000_DATASETS_PATH=/data/AE/modelzoo/datasets/aflw2000
export ICDAR_DATASETS_PATH=/data/AE/modelzoo/datasets/icdar2015
export HOIA_DATASETS_PATH=/data/AE/modelzoo/datasets/hoia/
export VOC2007_DATASETS_PATH=/data/AE/modelzoo/datasets/voc2007
export NNUNET_DATASETS_PATH=/data/AE/modelzoo/datasets/nnUNet_dataset
export MSRA_B_DATASETS_PATH=/data/AE/modelzoo/datasets/MSRA-B
export CRITEO_DATASETS_PATH=/data/AE/modelzoo/datasets/criteo
